package fr.insa.clavardator.ui;

import fr.insa.clavardator.users.CurrentUser;
import javafx.fxml.FXML;
import javafx.fxml.Initializable;
import javafx.scene.control.Label;

import java.beans.PropertyChangeEvent;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;
import java.util.ResourceBundle;

public class ToolbarController implements Initializable {

	@FXML
	private Label currentUsernameLabel;

	private List<ButtonPressEvent> editListeners;
	private List<ButtonPressEvent> aboutListeners;

	public void addEditListener(ButtonPressEvent listener) {
		editListeners.add(listener);
	}
	public void addAboutListener(ButtonPressEvent listener) {
		aboutListeners.add(listener);
	}

	public void onEditPress() {
		editListeners.forEach(ButtonPressEvent::onPress);
	}

	public void onAboutPress() {
		aboutListeners.forEach(ButtonPressEvent::onPress);
	}

	private void onUsernameChange(PropertyChangeEvent propertyChangeEvent) {
		if (propertyChangeEvent.getPropertyName().equals("username")) {
			final String newUsername = (String) propertyChangeEvent.getNewValue();
			currentUsernameLabel.setText(newUsername);
		}
	}

	@Override
	public void initialize(URL location, ResourceBundle resources) {
		CurrentUser.getInstance().addObserver(this::onUsernameChange);
		editListeners = new ArrayList<>();
		aboutListeners = new ArrayList<>();
	}
}
