package fr.insa.clavardator.ui.chat;

import fr.insa.clavardator.chat.ChatHistory;
import fr.insa.clavardator.chat.Message;
import fr.insa.clavardator.ui.ButtonPressEvent;
import fr.insa.clavardator.ui.LoadingScreenController;
import fr.insa.clavardator.ui.NoSelectionModel;
import fr.insa.clavardator.users.PeerUser;
import fr.insa.clavardator.util.ErrorCallback;
import fr.insa.clavardator.util.Log;
import javafx.application.Platform;
import javafx.collections.FXCollections;
import javafx.fxml.FXML;
import javafx.fxml.Initializable;
import javafx.scene.control.ListView;
import javafx.scene.layout.VBox;

import java.net.URL;
import java.util.ArrayList;
import java.util.ResourceBundle;

public class ChatController implements Initializable {

	@FXML
	private ListView<Message> messageList;
	@FXML
	private ChatFooterController chatFooterController;
	@FXML
	private ChatHeaderController chatHeaderController;
	@FXML
	private LoadingScreenController loadingController;

	@FXML
	private VBox chatContainer;
	@FXML
	private VBox emptyContainer;

	private PeerUser remoteUser;

	public void addAttachmentListener(ButtonPressEvent listener) {
		chatFooterController.addAttachmentListener(listener);
	}

	public void addSendErrorListener(ErrorCallback listener) {
		chatFooterController.addSendErrorListener(listener);
	}

	public void setRemoteUser(PeerUser remoteUser) {
		this.remoteUser = remoteUser;
		this.chatFooterController.setRemoteUser(remoteUser);
		this.chatHeaderController.setRemoteUser(remoteUser);
		setState(State.LOADING);

		final ChatHistory history = remoteUser.getHistory();
		history.addHistoryLoadedListener(this::onHistoryLoaded);
		history.addMessageAddedListener(this::onMessageAdded);
		history.load();
	}

	private void onHistoryLoaded(PeerUser user, ArrayList<Message> messages) {
		if (user.equals(remoteUser)) {
			messageList.setItems(FXCollections.observableArrayList(messages));
			messageList.scrollTo(messageList.getItems().size() - 1);
			setState(State.DONE);
		}
	}

	private void onMessageAdded(PeerUser user, Message message) {
		Log.v(this.getClass().getSimpleName(), "Message added: " + message.getText());
		Platform.runLater(() -> {
			messageList.getItems().add(message);
			messageList.scrollTo(messageList.getItems().size() - 1);
		});
	}

	private void setState(State state) {
		switch (state) {
			case INITIAL:
				emptyContainer.setVisible(true);
				chatFooterController.setEnabled(false);
				chatContainer.setVisible(false);
				loadingController.hide();
				messageList.setItems(null);
				break;
			case LOADING:
				chatContainer.setVisible(true);
				loadingController.show();
				chatFooterController.setEnabled(false);
				emptyContainer.setVisible(false);
				break;
			case DONE:
				chatFooterController.setEnabled(true);
				chatContainer.setVisible(true);
				emptyContainer.setVisible(false);
				loadingController.hide();
				break;
		}
	}

	@Override
	public void initialize(URL url, ResourceBundle rb) {
		loadingController.getRootStyle().clear();
		loadingController.getRootStyle().add("inner");
		messageList.setSelectionModel(new NoSelectionModel<>());
		messageList.setCellFactory(listView -> new MessageListItemCell());
		setState(State.INITIAL);
	}

	private enum State {
		INITIAL,
		LOADING,
		DONE
	}

}