package fr.insa.clavardator.ui.chat;

import com.jfoenix.controls.JFXTextField;
import fr.insa.clavardator.ui.ButtonPressEvent;
import fr.insa.clavardator.users.PeerUser;
import fr.insa.clavardator.util.ErrorCallback;
import fr.insa.clavardator.util.Log;
import javafx.beans.value.ObservableValue;
import javafx.fxml.FXML;
import javafx.fxml.Initializable;
import javafx.scene.input.KeyCode;
import javafx.scene.layout.HBox;

import java.net.URL;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.ResourceBundle;

public class ChatFooterController implements Initializable {

	@FXML
	private HBox container;
	@FXML
	private JFXTextField textField;

	private List<ButtonPressEvent> attachmentListeners;
	private List<ErrorCallback> sendErrorListeners;

	private PeerUser remoteUser;
	private HashMap<PeerUser, String> savedText;


	public void addAttachmentListener(ButtonPressEvent listener) {
		attachmentListeners.add(listener);
	}
	public void addSendErrorListener(ErrorCallback listener) {
		sendErrorListeners.add(listener);
	}

	public void onAttachmentPress() {
		attachmentListeners.forEach(ButtonPressEvent::onPress);
	}
	public void onSend() {
		if (remoteUser != null) {
			remoteUser.sendTextMessage(textField.getText(), this::onSendError);
		} else {
			Log.e(this.getClass().getSimpleName(), "Error: remote user not set");
		}
		textField.setText("");
	}

	public void onSendError(Exception e) {
		Log.e(this.getClass().getSimpleName(), "Error: Could not send message", e);
		sendErrorListeners.forEach((l) -> l.onError(e));
	}

	public void setEnabled(boolean enabled) {
		container.setDisable(!enabled);
	}

	private String findSavedText() {
		String text = null;
		if (remoteUser != null) {
			text = savedText.get(remoteUser);
		}
		return text != null ? text : "";
	}

	private void saveText(String text) {
		if (remoteUser != null) {
			savedText.put(remoteUser, text);
		}
	}

	public void onTextChange(ObservableValue<? extends String> observable, String oldText, String newText) {
		saveText(newText);
	}

	@Override
	public void initialize(URL location, ResourceBundle resources) {
		savedText = new HashMap<>();
		attachmentListeners = new ArrayList<>();
		sendErrorListeners = new ArrayList<>();
		textField.textProperty().addListener(this::onTextChange);
		textField.setOnKeyPressed(event -> {
			if (event.getCode() == KeyCode.ENTER) {
				event.consume();
				if(!textField.getText().isEmpty()){
					onSend();
				}
			}
		});
	}

	public void setRemoteUser(PeerUser remoteUser) {
		this.remoteUser = remoteUser;
		textField.setText(findSavedText());
	}

	public interface SendMessageEvent {
		void onSend(String text);
	}
}
