package fr.insa.clavardator.ui.dialogs;

import com.jfoenix.controls.JFXButton;
import com.jfoenix.controls.JFXDialog;
import com.jfoenix.controls.JFXTextField;
import com.jfoenix.validation.base.ValidatorBase;
import fr.insa.clavardator.ui.ButtonPressEvent;
import fr.insa.clavardator.users.CurrentUser;
import javafx.beans.property.ReadOnlyBooleanWrapper;
import javafx.beans.value.ObservableValue;
import javafx.fxml.FXML;
import javafx.fxml.Initializable;
import javafx.scene.control.Label;
import javafx.scene.input.KeyCode;
import javafx.scene.layout.StackPane;

import java.net.URL;
import java.util.ResourceBundle;
import java.util.Timer;
import java.util.TimerTask;

public class EditUsernameDialogController implements Initializable {

	enum State {
		VALID,
		INVALID,
		NETWORK
	}

	@FXML
	private Label titleLabel;
	@FXML
	private Label descriptionLabel;
	@FXML
	private JFXDialog dialog;
	@FXML
	private JFXTextField textField;
	@FXML
	private JFXButton confirmButton;
	@FXML
	private JFXButton cancelButton;

	private State currentState;
	private Validator validator;
	private ButtonPressEvent successListener;
	private ButtonPressEvent cancelListener;

	@FXML
	private void onConfirm() {
		setLocked(true);
		CurrentUser.getInstance().setUsername(textField.getText());
		// TODO replace by network call
		Timer t = new Timer();
		t.schedule(new TimerTask() {
			@Override
			public void run() {
//				onError();
				onSuccess();
				t.cancel();
			}
		}, 1000);
	}

	@FXML
	private void onCancel() {
		if (cancelListener != null) {
			cancelListener.onPress();
		}
		hide();
	}

	public void setOnSuccessListener(ButtonPressEvent listener) {
		this.successListener = listener;
	}
	public void setOnCancelListener(ButtonPressEvent listener) {
		this.cancelListener = listener;
	}

	public void show(StackPane root, Mode mode) {
		setLocked(false);
		setFieldError(State.VALID);
		setMode(mode);
		textField.setText("");
		dialog.show(root);
	}

	public void hide() {
		dialog.close();
	}

	private void setLocked(boolean state) {
		confirmButton.setDisable(state || textField.getText().isEmpty());
		cancelButton.setDisable(state);
		dialog.setOverlayClose(!state);
		textField.setDisable(state);
	}

	private void setFieldError(State state) {
		currentState = state;
		textField.validate();
	}

	public void onUsernameChange(ObservableValue<? extends String> observable, String oldText, String newText) {
		setFieldError(State.VALID);
		confirmButton.setDisable(newText.isEmpty());
	}

	public void setMode(Mode mode) {
		switch (mode) {
			case INITIAL:
				dialog.setOverlayClose(false);
				cancelButton.setText("Quitter");
				titleLabel.setText("Nom d'utilisateur");
				descriptionLabel.setText("Choisissez votre nom d'utilisateur.\nVous pourrez le changer plus tard");
				break;
			case ERROR:
				dialog.setOverlayClose(false);
				cancelButton.setText("Hors ligne");
				titleLabel.setText("Nom d'utilisateur expiré");
				descriptionLabel.setText("Votre nom d'utilisateur n'est plus valide.\nMerci d'en choisir un autre.");
				break;
			case EDIT:
				dialog.setOverlayClose(true);
				cancelButton.setText("Annuler");
				titleLabel.setText("Changer le nom d'utilisateur");
				descriptionLabel.setText("Entrez votre nouveau nom d'utilisateur");
				break;
		}
	}

	@Override
	public void initialize(URL location, ResourceBundle resources) {
		validator = new Validator();
		currentState = State.VALID;
		setMode(Mode.INITIAL);
		textField.textProperty().addListener(this::onUsernameChange);
		textField.getValidators().add(validator);
		textField.setOnKeyPressed(event -> {
			if (event.getCode() == KeyCode.ENTER) {
				event.consume();
				if(!textField.getText().isEmpty()){
					onConfirm();
				}
			}
		});
	}

	private void onSuccess() {
		setFieldError(State.VALID);
		setLocked(false);
		hide();
		if (successListener != null) {
			successListener.onPress();
		}
	}

	private void onError() {
		validator.setMessage("Nom d'utilisateur invalide");
		setFieldError(State.INVALID);
		setLocked(false);
	}


	public enum Mode {
		INITIAL,
		ERROR,
		EDIT
	}

	class Validator extends ValidatorBase {
		@Override
		protected void eval() {
			this.hasErrors = new ReadOnlyBooleanWrapper(currentState != State.VALID);
		}
	}
}
