package fr.insa.clavardator.ui.users;

import fr.insa.clavardator.ui.ButtonPressEvent;
import fr.insa.clavardator.ui.UserSelectedEvent;
import fr.insa.clavardator.users.PeerUser;
import fr.insa.clavardator.users.User;
import fr.insa.clavardator.users.UserList;
import fr.insa.clavardator.util.Log;
import javafx.application.Platform;
import javafx.fxml.FXML;
import javafx.fxml.Initializable;
import javafx.scene.control.ListView;

import java.net.URL;
import java.util.ResourceBundle;

public class UserListController implements Initializable {

	private ButtonPressEvent refreshUserListener;
	private UserSelectedEvent userSelectedListener;
	@FXML
	private ListView<PeerUser> peerUserListView;
	private UserList userList;

	public UserListController() {
	}

	public void setRefreshUserListener(ButtonPressEvent listener) {
		refreshUserListener = listener;
	}

	public void setUserSelectedListener(UserSelectedEvent listener) {
		userSelectedListener = listener;
	}

	public void onRefreshUserListPress() {
		refreshUserListener.onPress();
	}

	private void onUserSelected(@org.jetbrains.annotations.NotNull PeerUser user) {
		final User currentSelectedUser = peerUserListView.getSelectionModel().getSelectedItem();
		if (!user.equals(currentSelectedUser)) {
			peerUserListView.getSelectionModel().select(user);
			userSelectedListener.onSelected(user);
		}
	}

	@Override
	public void initialize(URL url, ResourceBundle rb) {
		peerUserListView.setCellFactory(listView -> {
			final UserListItemCell cell = new UserListItemCell();
			cell.setOnUserSelectedListener(this::onUserSelected);
			return cell;
		});
	}

	private void onUserConnected(PeerUser user) {
		Log.v(this.getClass().getSimpleName(), "Add user to UI");
		Platform.runLater(() -> peerUserListView.getItems().add(user));
	}

	public void setUserList(UserList userList) {
		this.userList = userList;
		userList.addNewUserObserver(this::onUserConnected);
	}
}