package fr.insa.clavardator.ui.users;

import com.jfoenix.controls.JFXButton;
import fr.insa.clavardator.ui.ButtonPressEvent;
import fr.insa.clavardator.users.PeerUser;
import javafx.application.Platform;
import javafx.fxml.FXML;
import javafx.fxml.Initializable;

import java.beans.PropertyChangeEvent;
import java.net.URL;
import java.util.ResourceBundle;

public class UserListItemController implements Initializable {

	@FXML
	private JFXButton button;
	@FXML
	private UserActiveIndicatorController indicatorController;

	private ButtonPressEvent listener;

	private PeerUser user;

	public void setOnPressListener(ButtonPressEvent listener) {
		this.listener = listener;
	}

	public void onPress() {
		if (listener != null) {
			listener.onPress();
		}
	}

	public void setSelected(boolean selected) {
		if (selected)
			setBackgroundSelected();
		else
			resetBackground();
	}

	@Override
	public void initialize(URL location, ResourceBundle resources) {
		resetBackground();
	}


	private void onUsernameChange(PropertyChangeEvent propertyChangeEvent) {
		if (propertyChangeEvent.getPropertyName().equals("username")) {
			final String newUsername = (String) propertyChangeEvent.getNewValue();
			Platform.runLater(() -> button.setText(newUsername));
		}
	}

	public void setUser(PeerUser user) {
		if (this.user != null) {
			// remove old observer before setting new user
			this.user.removeObserver(this::onUsernameChange);
		}
		this.user = user;
		user.addObserver(this::onUsernameChange);
		indicatorController.setUser(user);
		button.setText(user.getUsername());
		resetBackground();
	}

	private void resetBackground() {
		if (user != null && user.isActive()) {
			button.getStyleClass().remove("inactive-user-item");
			button.getStyleClass().add("active-user-item");
		} else {
			button.getStyleClass().remove("active-user-item");
			button.getStyleClass().add("inactive-user-item");
		}
	}

	private void setBackgroundSelected() {
		button.getStyleClass().remove("inactive-user-item");
		button.getStyleClass().remove("active-user-item");
		button.getStyleClass().add("selected-user-item");
	}
}
