package fr.insa.clavardator.users;

import fr.insa.clavardator.network.NetUtil;
import fr.insa.clavardator.util.Log;
import javafx.application.Platform;

import java.net.InetAddress;
import java.net.SocketException;
import java.util.List;
import java.util.Timer;
import java.util.TimerTask;

import static fr.insa.clavardator.network.NetUtil.getIdFromIp;

public class CurrentUser extends User {
	private static CurrentUser instance = null;

	private State state;

	private CurrentUser() {
		super();
		state = State.UNINITIALIZED;
	}

	/**
	 * Returns the current.
	 */
	public static CurrentUser getInstance() {
		if (instance == null)
			instance = new CurrentUser();
		return instance;
	}

	public void init() throws SocketException {
		final List<InetAddress> addresses = NetUtil.listAllLocalAddresses();
		if (addresses.size() > 0) {
			id = getIdFromIp(addresses.get(0));
		} else {
			throw new SocketException();
		}
		// TODO replace by db username fetching
		Timer t = new Timer();
		t.schedule(new TimerTask() {
			@Override
			public void run() {
				setState(State.NONE);
				t.cancel();
			}
		}, 500);
	}

	@Override
	public void setUsername(String newUsername) {
		Log.v(this.getClass().getSimpleName(),
				"Username changed from " + getUsername() + " to " + newUsername);
		super.setUsername(newUsername);
		setState(State.VALID);
	}

	public void setState(State state) {
		Log.v(this.getClass().getSimpleName(),
				"State changed from " + this.state.toString() + " to " + state.toString());
		instance.pcs.firePropertyChange("state", this.state, state);
		this.state = state;
	}

	public enum State {
		UNINITIALIZED,
		VALID,
		INVALID,
		NONE
	}
}
